import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';

class TerminalImage {
  String? previewUrl;
  String? imageUrl;
  String? coverImageUrl;

  TerminalImage({
    this.previewUrl,
    this.imageUrl,
    this.coverImageUrl,
  });

  factory TerminalImage.fromJson(Map<String, dynamic> map) {
    return TerminalImage(
      previewUrl: map['PreviewUrl'],
      imageUrl: map['ImageUrl'],
      coverImageUrl: map['CoverImageUrl'],
    );
  }

  Map<String, dynamic> toJson() {
    final map = Map<String, dynamic>();
    if (previewUrl != null) map['PreviewUrl'] = previewUrl;
    if (imageUrl != null) map['ImageUrl'] = imageUrl;
    if (coverImageUrl != null) map['CoverImageUrl'] = coverImageUrl;
    return map;
  }
}

/// 所有图片的入口
class ContentImage extends StatelessWidget implements FWidget {
  /// 图片类型
  final RemedicalFileDataTypeEnum fileDataType;

  /// 图片地址
  final TerminalImage terminalImage;

  /// AI 良恶性
  final DiagnosisConclusionEnum? diagnosisConclusion;

  /// 是否是测量页面
  final bool? isMeasure;

  /// 改变图片地址
  final VoidCallback? onChangeImage;

  /// AI测量的器官
  final List<DiagnosisOrganEnum>? diagnosisOrgans;

  /// 判断是否有AI器官
  bool get hasDiagnosisOrgans => diagnosisOrgans?.length != 0;

  ContentImage({
    Key? key,
    required this.fileDataType,
    required this.terminalImage,
    this.diagnosisConclusion = DiagnosisConclusionEnum.NotRequired,
    this.diagnosisOrgans = const [DiagnosisOrganEnum.Null],
    this.isMeasure = false,
    this.onChangeImage,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return _buildContentImage(
      fileDataType,
      terminalImage.previewUrl!,
    );
  }

  /// 0
  Widget _buildContentImage(
    RemedicalFileDataTypeEnum fileDataType,
    String previewUrl,
  ) {
    switch (fileDataType) {
      case RemedicalFileDataTypeEnum.Image:
      case RemedicalFileDataTypeEnum.ThirdVidSingle:
      case RemedicalFileDataTypeEnum.VinnoVidSingle:
        return _buildImageCard(
          previewUrl,
        );
      case RemedicalFileDataTypeEnum.ThirdVidMovie:
      case RemedicalFileDataTypeEnum.VinnoVidMovie:
        return _buildVidMovieCard(
          previewUrl,
        );
      default:
        return Container(
          child: Text('error'),
        );
    }
  }

  Widget _buildVidMovieCard(
    String previewUrl,
  ) {
    return Listener(
      child: Container(
        width: 190,
        height: 160,
        color: Colors.black,
        child: Stack(
          children: [
            Center(
              child: Image.network(
                previewUrl,
                errorBuilder: ((context, error, stackTrace) {
                  return Container(
                    child: Text('error'),
                  );
                }),
              ),
            ),
            hasDiagnosisOrgans
                ? Positioned(
                    bottom: 5,
                    left: 5,
                    child: _buildAIDiagnosticOrgans(),
                  )
                : const SizedBox(),
            InkWell(
              onTap: () {
                onChangeImage!.call();
              },
              child: Center(
                child: Container(
                  child: Icon(
                    Icons.play_circle_outline_rounded,
                    color: Colors.white,
                    size: 50,
                  ),
                ),
              ),
            ),
            Positioned(
              bottom: 5,
              right: 5,
              child: Tooltip(
                textStyle: TextStyle(
                  fontSize: 16,
                  color: Colors.white,
                ),
                message: _buildAIText(diagnosisConclusion!),
                child: Text(
                  'AI',
                  style: TextStyle(
                    color: _buildAITextColor(
                      diagnosisConclusion!,
                    ),
                    fontSize: 20,
                  ),
                ),
              ),
            ),
          ],
        ),
      ),
    );
  }

  Widget _buildImageCard(
    String previewUrl,
  ) {
    return Listener(
      child: InkWell(
        onTap: () {
          onChangeImage!.call();
        },
        child: Container(
          width: 190,
          height: 160,
          color: Colors.black,
          child: Stack(
            children: [
              Center(
                child: Image.network(
                  previewUrl,
                  errorBuilder: ((context, error, stackTrace) {
                    return Container(
                      child: Text('error'),
                    );
                  }),
                ),
              ),
              hasDiagnosisOrgans
                  ? Positioned(
                      bottom: 5,
                      left: 5,
                      child: _buildAIDiagnosticOrgans(),
                    )
                  : const SizedBox(),
              Positioned(
                bottom: 5,
                right: 5,
                child: Tooltip(
                  textStyle: TextStyle(
                    fontSize: 16,
                    color: Colors.white,
                  ),
                  message: _buildAIText(diagnosisConclusion!),
                  child: Text(
                    'AI',
                    style: TextStyle(
                      color: _buildAITextColor(
                        diagnosisConclusion!,
                      ),
                      fontSize: 20,
                    ),
                  ),
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }

  String _buildAIText(DiagnosisConclusionEnum diagnosisConclusion) {
    switch (diagnosisConclusion) {
      case DiagnosisConclusionEnum.NotRequired:
        return i18nBook.remedical.noNeedAIYet.t;
      case DiagnosisConclusionEnum.InProcess:
        return i18nBook.remedical.didNotHandle.t;
      case DiagnosisConclusionEnum.Unrecognized:
        return i18nBook.remedical.didNotIdentify.t;
      case DiagnosisConclusionEnum.NoObviousLesion:
        return i18nBook.remedical.noAbnormal.t;
      case DiagnosisConclusionEnum.Benign:
        return i18nBook.remedical.benign.t;
      case DiagnosisConclusionEnum.Malignant:
        return i18nBook.remedical.malignant.t;
      case DiagnosisConclusionEnum.BenignAndMalignant:
        return i18nBook.remedical.benignWithMalignant.t;
      default:
        return i18nBook.remedical.didNotIdentify.t;
    }
  }

  dynamic _buildAITextColor(DiagnosisConclusionEnum diagnosisConclusion) {
    switch (diagnosisConclusion) {
      case DiagnosisConclusionEnum.Unrecognized:
        return Colors.grey;
      case DiagnosisConclusionEnum.NoObviousLesion:
        return Colors.lightBlue;
      case DiagnosisConclusionEnum.Benign:
        return Colors.greenAccent;
      case DiagnosisConclusionEnum.Malignant:
        return Colors.redAccent;
      case DiagnosisConclusionEnum.BenignAndMalignant:
        return Colors.orangeAccent;
      default:
        return Colors.transparent;
    }
  }

  Widget _buildAIDiagnosticOrgans() {
    switch (diagnosisOrgans?[0]) {
      case DiagnosisOrganEnum.Null:
        return const SizedBox();
      case DiagnosisOrganEnum.placeHolder_1:
        return const SizedBox();
      case DiagnosisOrganEnum.Breast:
        return FIcon(
          FIcons.fis_breast,
          color: _buildAITextColor(
            diagnosisConclusion!,
          ),
        );
      case DiagnosisOrganEnum.Abdomen:
        return FIcon(
          FIcons.fis_abdomen,
          color: _buildAITextColor(
            diagnosisConclusion!,
          ),
        );

      case DiagnosisOrganEnum.Liver:
        return FIcon(
          FIcons.fis_liver,
          color: _buildAITextColor(
            diagnosisConclusion!,
          ),
        );
      case DiagnosisOrganEnum.Cholecyst:
        return FIcon(
          FIcons.fis_gallbladder,
          color: _buildAITextColor(
            diagnosisConclusion!,
          ),
        );
      case DiagnosisOrganEnum.Kidney:
        return FIcon(
          FIcons.fis_kidney,
          color: _buildAITextColor(
            diagnosisConclusion!,
          ),
        );
      case DiagnosisOrganEnum.Spleen:
        return FIcon(
          FIcons.fis_spleen,
          color: _buildAITextColor(
            diagnosisConclusion!,
          ),
        );

      default:
        return Container();
    }
  }
}
